﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Drawing;

namespace GDIEngine
{
    

    public class Engine : Control , IDisposable
    {


        #region Delegates
        
        public delegate void GameDraw();

        #endregion


        #region Events

        public virtual event GameDraw OnGameDraw;

        #endregion

        
        #region Public Field

        /// <summary>
        /// Device Draw Engine
        /// </summary>
        public Graphics device { get; set; }

        /// <summary>
        /// Viewport to World Transform Position
        /// </summary>
        public Camera2D Camera = new Camera2D(0f, 0f);


        /// <summary>
        /// Quality Draw Setting
        /// </summary>
        public QualitySetting DeviceQualityRendering = QualitySetting.None;

        /// <summary>
        /// Clear Color
        /// </summary>
        public Color ClearColor = Color.Black;


        /// <summary>
        /// The Engine Content Manager
        /// </summary>
        public AssetManager ContentManager = new AssetManager();

        public bool FullScreen
        {
            get { return fscreen; }
            set { fscreen = value; UpdateSetting(); }
        }

        /// <summary>
        /// Zoom Scale from Scene
        /// </summary>
        public float Zoom
        {
            get { return zoom; }
            set { zoom = value; }
        }

        #endregion

        #region Private Field

        private Timer time = new Timer();

        bool antialiasing = false;
        bool LowQuality = false;
        bool MediumQuality = false;
        bool HighQuality = false;

        bool debug = false;
        bool fscreen = false;
        float zoom = 1f;

        #endregion


        #region Public Struct

        


        #endregion


        #region Public Enum

        public enum QualitySetting 
        {
            None = 1 << 0,
            Antialasing = 1 << 1,
            LowQuality = 1 << 2,
            MediumQuality = 1 << 3,
            HighQuality = 1 << 4
        }

        #endregion

    

        /// <summary>
        /// Constructor
        /// </summary>
        public Engine()
        {
            SetStyle(ControlStyles.AllPaintingInWmPaint | ControlStyles.UserMouse | ControlStyles.UserPaint | ControlStyles.OptimizedDoubleBuffer | ControlStyles.SupportsTransparentBackColor, true);
            Dock = DockStyle.Fill;
            OnGameDraw += new GameDraw(Engine_OnGameDraw);

            time.Interval = 25;
            time.Tick += new EventHandler(time_Tick);
            time.Start();

        }


        public void UpdateSetting()
        {
            if (FullScreen)
            {
                FindForm().WindowState = FormWindowState.Normal;
                FindForm().FormBorderStyle = FormBorderStyle.None;
                FindForm().WindowState = FormWindowState.Maximized;
                FindForm().Size = Screen.PrimaryScreen.WorkingArea.Size;
                FindForm().BringToFront();
            }
        }


        protected override void OnHandleCreated(EventArgs e)
        {
            base.OnHandleCreated(e);

          
        }


        #region Internal void

        internal void time_Tick(object sender, EventArgs e)
        {
            Invalidate(false);
        }

        internal void Engine_OnGameDraw()
        {
            
        }

        #endregion


        protected override void OnPaint(PaintEventArgs e)
        {
            base.OnPaint(e);
            FindForm().SuspendLayout();

            device = e.Graphics;

            device.Clear(ClearColor);

            #region Quality Setting
            uint value = (uint)DeviceQualityRendering;

            switch (value)
            {
                case 1:          
                    break;
                case 2:
                     antialiasing = true;
                break;

                case 3:
                    LowQuality = true;
                break;
                case 4:
                  MediumQuality = true;
                break;
                case 5:
                    HighQuality = true;
                break;
                case 6:
                antialiasing = true;
                    LowQuality = true;
                break;
                case 10:
                    antialiasing = true;
                    MediumQuality = true;
                break;
                case 18:
                    antialiasing = true;
                    HighQuality = true;
                break;

            }

            #endregion


            if (antialiasing)
                device.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias;
            else if (antialiasing && LowQuality)
                device.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias | System.Drawing.Drawing2D.SmoothingMode.HighSpeed;
            else if (antialiasing && MediumQuality)
                device.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias | System.Drawing.Drawing2D.SmoothingMode.Default;
            else if (antialiasing && HighQuality)
                device.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias | System.Drawing.Drawing2D.SmoothingMode.HighQuality;
            else if (LowQuality)
            {
                device.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.HighSpeed;
            }

            device.TranslateTransform(Camera.X, Camera.Y);

            device.ScaleTransform(Zoom, Zoom);

           OnGameDraw();
         

                        
            
            FindForm().ResumeLayout();
        }


        #region DrawString

        /// <summary>
        /// Draw simple string whit default System Font
        /// </summary>
        /// <param name="Text">Text</param>
        /// <param name="position">Position</param>
        public void DrawString(string Text,Vector2 position)
        {
            device.DrawString(Text, new Font("Arial", 24), Brushes.White, new PointF(position.X, position.Y));
        }

        /// <summary>
        /// Draw simple string whit default System Font
        /// </summary>
        /// <param name="Text">Text</param>
        /// <param name="position">Position</param>
        /// <param name="TextColor">Text Color</param>
        public void DrawString(string Text, Vector2 position, Color TextColor)
        {
            device.DrawString(Text, new Font("Arial", 24), new SolidBrush(TextColor), new PointF(position.X, position.Y));
        }


        /// <summary>
        /// Draw simple string whit default System Font
        /// </summary>
        /// <param name="Text">Text</param>
        /// <param name="position">Position</param>
        /// <param name="FontSize">Font Size</param>
        public void DrawString(string Text, Vector2 position, Color TextColor, int FontSize)
        {
            device.DrawString(Text, new Font("Arial", FontSize), new SolidBrush(TextColor), new PointF(position.X, position.Y));
        }


        /// <summary>
        /// Draw simple string whit default System Font
        /// </summary>
        /// <param name="Text">Text</param>
        /// <param name="position">Position</param>
        /// <param name="FontSize">Font Size</param>
        /// <param name="FontName">Font Name</param>
        public void DrawString(string Text, Vector2 position, Color TextColor,string FontName, int FontSize)
        {
            device.DrawString(Text, new Font(FontName, FontSize), new SolidBrush(TextColor), new PointF(position.X, position.Y));
        }

        #endregion


        /// <summary>
        /// DrawTexture
        /// </summary>
        /// <param name="Texture">Texture</param>
        /// <param name="position">Position</param>
        public void DrawTexture(Texture2D Texture,Vector2 position)
        {
            device.DrawImage(Texture.GetImage(), new Point(position.X, position.Y));
        }


        /// <summary>
        /// Draw Texture
        /// </summary>
        /// <param name="TiledTexture">Tiled Texture</param>
        /// <param name="Frame">Frame ID</param>
        /// <param name="position">Position</param>
        public void DrawTexture(TiledTexture TiledTexture, int Frame, Vector2 position)
        {            
             device.DrawImage(TiledTexture.GetImage(),position.X,position.Y,TiledTexture.Frame[Frame], GraphicsUnit.Pixel);
        }



        public void PlayAnimatedTexture(AnimatedTexture text, Vector2 position,bool oneShoot = true)
        {
            int frame=0,max = text.Frame.Count;
            
       
     
            for (int i = 0; i < max; i++)
            {
                device.DrawImage(text.GetImage(),position.X,position.Y,text.Frame[0],GraphicsUnit.Pixel);

            }
   
        }
    
    }

}
