﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Drawing;

namespace GDIEngine
{
    public sealed class Texture2D
    {

        //Byte Array struct from Image file
        private byte[] Data;


        /// <summary>
        /// Load Texture Data From File
        /// </summary>
        /// <param name="file_location"></param>
        public Texture2D(string file_location)
        {          
            Data = File.ReadAllBytes(file_location);
        }



        /// <summary>
        /// Get Image format from Texture Data
        /// </summary>
        /// <returns>return image from Texture</returns>
        public Image GetImage()
        {
            try
            {
                MemoryStream ms = new MemoryStream(Data, 0, Data.Length);
                ms.Write(Data, 0, Data.Length);
                return Image.FromStream(ms);
            }
            catch 
            {
                throw new InvalidDataException();           
            }
        }



        private byte[] ImageToByteArray(Image imageIn)
        {
            using (var ms = new MemoryStream())
            {
                imageIn.Save(ms, imageIn.RawFormat);
                return ms.ToArray();
            }
        }


        /// <summary>
        /// Set alpha transparency color
        /// </summary>
        /// <param name="c">Color</param>
        public void SetAlphaColor(Color c)
        {
            Bitmap b = (Bitmap)GetImage();
            b.MakeTransparent(c);

            Data = ImageToByteArray(b);
        }

    }
}
