﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Drawing;

namespace GDIEngine
{
    public class TiledTexture
    {

        private byte[] Data;

        public List<Rectangle> Frame = new List<Rectangle>();

        /// <summary>
        /// Load a TiledTexture From File
        /// </summary>
        /// <param name="file">file path</param>
        public TiledTexture(string file,int HorizzontalFrame,int VerticalFrame)
        {
            Data = File.ReadAllBytes(file);
            Image text = GetImage();

            for (int i = 0; i < VerticalFrame * (text.Height / VerticalFrame); i += (text.Height / VerticalFrame))
            {
                for (int j = 0; j < HorizzontalFrame * (text.Width / HorizzontalFrame); j += (text.Width / HorizzontalFrame))
                {
                    Frame.Add(new Rectangle(j, i, (text.Width / HorizzontalFrame), (text.Height / VerticalFrame)));
                }
            }

          
        }



        /// <summary>
        /// Get Image format from Texture Data
        /// </summary>
        /// <returns>return image from Texture</returns>
        public Image GetImage()
        {
            try
            {
                MemoryStream ms = new MemoryStream(Data, 0, Data.Length);
                ms.Write(Data, 0, Data.Length);
                return Image.FromStream(ms);
            }
            catch
            {
                throw new InvalidDataException();
            }
        }



        private byte[] ImageToByteArray(Image imageIn)
        {
            using (var ms = new MemoryStream())
            {
                imageIn.Save(ms, imageIn.RawFormat);
                return ms.ToArray();
            }
        }


        /// <summary>
        /// Set alpha transparency color
        /// </summary>
        /// <param name="c">Color</param>
        public void SetAlphaColor(Color c)
        {
            Bitmap b = (Bitmap)GetImage();
            b.MakeTransparent(c);

            Data = ImageToByteArray(b);
        }

        public void AutoAlpha()
        {
            Bitmap b = (Bitmap)GetImage();
            Color c = b.GetPixel(0, 0);
            b.MakeTransparent(c);
            Data = ImageToByteArray(b);
        }
    }
}
